# -*- coding: utf-8 -*-

from base64 import b64encode
from datetime import datetime
from json import JSONEncoder
import itertools

from django.conf import settings
from django.contrib import messages
from django.core.cache import cache
from django.core.mail import send_mail, EmailMultiAlternatives
from django.http import Http404
from django.shortcuts import get_object_or_404, render, redirect
from django.template.defaultfilters import slugify
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


from club.models import Club, ArticleBlog, AliasGere, JetonEdition, LISTE_STAFF, update_domaines
from ldap_inp import LdapEleve, LdapClub, LdapGroupe
from lib.access_auth import access_restricted, permission_needed, forbidden
from lib.tools import mail_root, confirmation, erreur
from edt.models import EdtClubs
from portailuser.models import Ecole
from portailuser.app_logger import AppLogger
from .forms import (
    NouvClubForm,
    AliasGereForm,
    ArticleBlogForm,
    ClubForm,
    ClubDescriptionForm,
    EdtClubsForm,
    LogoForm,
    DomaineForm,
)
from portailuser.models import PortailUser
from gestion.aliases.views import liste_inscrits
from gestion.aliases.views import search_login as search_login_alias
from gestion.aliases.models import AssociationAdresseUser


_logger = AppLogger("gestion_clubs")

DOMAINES_ASSOCIATION_INTERDITE = settings.DOMAINES_INSTITUTIONNELS + settings.DOMAINES_INP_NET


@permission_needed("gerer_clubs")
def liste(request):
    """
    Liste des clubs

    Accessible seulement à ceux pouvant gérer les clubs (gerer_clubs)
    """
    clubs = sorted(
        LdapClub.objects.filter(ecole=request.ecole),
        key=lambda c: c.displayName.lower(),
    )

    return render(request, "gestion/clubs/liste.html", {"clubs": clubs})


def infos_perms(request, club_id):
    """
    Retourne les infos concernant le club ``club_id`` :
    * l'objet ``Club``
    * les données du clubs (postes, noms, …)
    * les permissions liées au club
    """
    try:
        club = Club.objects.par_ident(request.ecole, club_id)
    except Club.DoesNotExist:
        raise Http404

    infos = club.get_infos()
    postes_bureau = [infos["president"]]
    postes_bureau.extend(infos["tresorier"])
    postes_bureau.extend(infos["vicePresident"])
    postes_bureau.extend(infos["secretaire"])

    a_perm_clubs = request.user.check_permission(request, "gerer_clubs")
    a_perm_aliases = request.user.check_permission(request, "gerer_aliases")
    ident_groupe = club_id + "-" + request.ecole.ident
    est_dans_club = bool(LdapClub.objects.filter(cn=ident_groupe, memberUid=request.user.username))
    est_dans_bureau = request.user.username in postes_bureau

    if not est_dans_club and not a_perm_clubs:
        return club, infos, None

    return (
        club,
        infos,
        {
            "bureau": a_perm_clubs or est_dans_bureau,
            "aliases": a_perm_aliases or est_dans_bureau,
            "membres": a_perm_clubs or est_dans_bureau,
            "blog": a_perm_clubs or est_dans_club,
            "aliases_geres": club.aliasgere_set.filter(confirme=True),
        },
    )


@access_restricted
def gest_accueil(request, club_id):
    """
    Gestion des permissions pour les différentes pages du club ``club_id``.
    Renvoie vers la page d'accueil du club, suivant les permissions de
    l'utilisateur.
    """

    club, infos, perms = infos_perms(request, club_id)

    if not perms:
        return forbidden(request)

    if perms["bureau"]:
        return redirect("gestion:clubs:gest_bureau", club_id=club_id)

    if perms["membres"]:
        return redirect("gestion:clubs:gest_membres", club_id=club_id)

    return redirect("gestion:clubs:gest_blog", club_id=club_id)


@access_restricted
def description(request, club_id):
    """
    Gestion de la description du club.
    """

    club, info, perms = infos_perms(request, club_id)

    if not perms or not perms["bureau"]:
        return forbidden(request)

    previs = None

    if request.method == "POST":
        form = ClubDescriptionForm(request.POST, instance=club)

        if "previs" in request.POST:
            previs = form.get_preview()

        elif form.is_valid():
            form.save()
            return redirect("gestion:clubs:gest_accueil", club_id=club_id)

    else:
        form = ClubDescriptionForm(instance=club)

    c = {
        "club": club,
        "perms": perms,
        "form": form,
        "previs": previs,
        "page_description": True,
    }

    return render(request, "gestion/clubs/description.html", c)


@access_restricted
def gest_bureau(request, club_id):
    """
    Gestion du bureau du club ``club_id``.
    Possible uniquement si l'utilisateur a le droit ``bureau`` (cf. info_perms).
    """
    club, infos, perms = infos_perms(request, club_id)
    est_bde = club_id == "bde"

    if not perms or not perms["bureau"]:
        return forbidden(request)

    if request.method == "POST" and "envoi_bureau" in request.POST:
        form = ClubForm(request.POST)
        if form.is_valid():
            club.blog_act = bool(form.cleaned_data["blog_act"])
            club.save()

            donnees_form = form.cleaned_data

            donnees = {
                "displayName": donnees_form["displayName"],
                "activite": donnees_form["activite"],
                "typeClub": donnees_form["typeClub"],
                "president": donnees_form["president"].username,
                "tresorier": donnees_form["tresorier"],
                "local": donnees_form["local"],
                "mailAlias": donnees_form["mailAlias"],
                "vicePresident": donnees_form["vicePresident"],
                "secretaire": donnees_form["secretaire"],
            }

            # vérification des cotisants AEn7
            if club.ecole == Ecole.n7:
                bureau = {donnees["president"]}
                bureau.update(donnees["tresorier"])
                bureau.update(donnees["vicePresident"])
                bureau.update(donnees["secretaire"])
                etus = [LdapEleve.objects.get(uid=uid) for uid in bureau]
                inscr = True
                for etu in etus:
                    if not etu.inscritAE:
                        inscr = False
                        messages.error(request, f"{etu.cn} n'est pas cotisant à l'AE")
                if not inscr:
                    return redirect("gestion:clubs:gest_bureau", club_id=club_id)

            passation, diff = club.maj_infos(donnees)
            # Mise en forme des informations pour l'affichage des données
            for idx in LISTE_STAFF:
                if idx in diff:
                    ancien, nouveau = diff[idx]
                    if isinstance(nouveau, str):
                        nouveau = [nouveau]
                    ancien = [LdapEleve.objects.get(uid=e) for e in ancien]
                    nouveau = [LdapEleve.objects.get(uid=e) for e in nouveau]
                    diff[idx] = (ancien, nouveau)

            if club.ecole == Ecole.n7 and diff:
                mail_from = settings.DEFAULT_FROM_EMAIL
                mail_to = ["respo_club@bde.enseeiht.fr"]
                sujet_mail = f"Modification du bureau de {club.nom}"
                c = {
                    "nom": club.nom,
                    "diff": diff,
                }
                contenu_mail = render_to_string("gestion/clubs/mail_respo_club.txt", c)
                msg = EmailMultiAlternatives(sujet_mail, contenu_mail, mail_from, mail_to)
                msg.attach(
                    f"Fiche de passation de {club.nom}.html",
                    fiche_passation(request, club_id, True),
                    "text/html",
                )
                msg.send()

            if passation and not est_bde:
                return redirect("gestion:clubs:fiche_passation", club_id=club_id)

            messages.success(request, "Informations du club modifiées avec succès.")
            return redirect("gestion:clubs:gest_bureau", club_id=club_id)
    else:
        init = dict(infos)
        init["blog_act"] = club.blog_act
        form = ClubForm(init)

    if perms["aliases"]:
        aliases = AliasGere.objects.filter(club=club)

        if request.method == "POST" and "aj_alias" in request.POST:
            alias_form = AliasGereForm(club, request.POST)

            if alias_form.is_valid():
                alias = alias_form.save(commit=False)
                if AliasGere.objects.filter(prefixe=alias.prefixe, club=club):
                    messages.error(request, "L'alias existe déjà.")
                    return redirect("gestion:clubs:gest_bureau", club_id=club_id)

                alias.club = club
                alias.save()

                messages.success(request, "Alias ajouté au club avec succès.")
                return redirect("gestion:clubs:gest_bureau", club_id=club_id)
        else:
            alias_form = AliasGereForm(club)

        aliases = AliasGere.objects.filter(club=club)

        if request.method == "POST" and "aj_domaine" in request.POST:
            domaines_form = DomaineForm(request.POST)

            if domaines_form.is_valid():
                ajouter_domaine(request, club, domaines_form.cleaned_data["domaine"])

                return redirect("gestion:clubs:gest_bureau", club_id=club_id)
        else:
            domaines_form = DomaineForm()
    else:
        aliases = None
        alias_form = None
        domaines_form = None

    # logo du club
    if request.method == "POST" and "aj_logo" in request.POST:
        logo_form = LogoForm(request.POST, request.FILES)

        if logo_form.is_valid():
            logo = logo_form.cleaned_data["logo"]
            club.logo = logo
            club.save()

            _logger.log(request.user, f"A modifié le logo du club {club_id}")
            messages.success(request, "Logo ajouté au club avec succès.")
            return redirect("gestion:clubs:gest_bureau", club_id=club_id)
    else:
        logo_form = LogoForm()

    c = {
        "club": club,
        "perms": perms,
        "form": form,
        "est_bde": est_bde,
        "page_bureau": True,
        "domaines": club.get_ldap_club().domainesAssocies,
        "aliases": aliases,
        "alias_form": alias_form,
        "domaines_form": domaines_form,
        "logo_form": logo_form,
    }

    return render(request, "gestion/clubs/gest_bureau.html", c)


@access_restricted
def gest_alias(request, club_id, prefixe):
    """
    Gestion des alias associés au club ``club_id``.
    Possible uniquement si l'utilisateur a le droit ``membres`` (cf. info_perms).
    """
    club, _, perms = infos_perms(request, club_id)

    if not perms or not perms["membres"]:
        return forbidden(request)

    alias = get_object_or_404(AliasGere, club=club, prefixe=prefixe, confirme=True)

    if request.method == "POST" and "adresse" in request.POST:
        liste = [adr for adr in request.POST.getlist("adresse")]
        alias_ldap = alias.alias_ldap
        for adr in liste:
            try:
                alias_ldap.rfc822MailMember.remove(adr)
            except ValueError:
                pass
            associations = AssociationAdresseUser.objects.filter(adresse=adr).filter(alias=alias_ldap.cn[0])
            for association in associations:
                association.delete()

        alias_ldap.save()

        messages.success(request, "La sélection a été supprimée de l’alias.")
        return redirect("gestion:clubs:gest_alias", club_id=club_id, prefixe=prefixe)

    c = {
        "club": club,
        "perms": perms,
        "inscrits": liste_inscrits(alias.alias_ldap),
        "alias": alias,
    }

    return render(request, "gestion/clubs/gest_alias.html", c)


@access_restricted
def suppr_alias(request, club_id, prefixe):
    """
    Supprimer la gestion de l'alias ``prefixe`` pour le club ``club_id``
    Possible uniquement si l'utilisateur a le droit ``bureau`` ou ``aliases``
    (cf. ``info_perms``)
    """
    club, _, perms = infos_perms(request, club_id)

    if not perms or not perms["bureau"] or not perms["aliases"]:
        return forbidden(request)

    alias = get_object_or_404(AliasGere, club=club, prefixe=prefixe)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            alias.delete()
        return redirect("gestion:clubs:gest_bureau", club_id=club_id)

    message = """Voulez-vous vraiment supprimer la gestion de cet alias ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer l'alias")]

    return confirmation(request, message, boutons)


def ajouter_domaine(request, club, domaine):
    """
    Ajoute un domaine à un club, si celui-ci n'est pas déjà utilisé.
    """

    domaine = domaine.lower()

    if domaine in DOMAINES_ASSOCIATION_INTERDITE:
        messages.error(
            request,
            "Ce nom de domaine est réservé et ne peut pas être associé à ce club, contactez INP-net si vous pensez que c'est une erreur",
        )
        return

    if domaine in club.get_ldap_club().domainesAssocies:
        messages.error(request, "Le domaine d'alias est déjà associé à ce groupe")
        return

    # On itère sur tous les domaines déjà associés (petite subtilité: LdapGroupe.domainesAssocies
    # est une liste, donc domainesUtilises ici est une liste de listes, d'où l'utilisation
    # de itertools)
    domainesUtilises = [groupe.domainesAssocies for groupe in LdapGroupe.objects.all()]
    if domaine in itertools.chain(*domainesUtilises):
        messages.error(request, "Ce domaine d'alias est déjà utilisé par un autre groupe")
        return

    ldap_club = club.get_ldap_club()
    ldap_club.domainesAssocies.append(domaine)
    ldap_club.save()

    update_domaines(ldap_club)

    messages.success(request, "Domaine d'alias ajouté au club avec succès.")


@access_restricted
def suppr_domaine(request, club_id, domaine):
    """
    Retire l'affiliation du domaine `domaine` et supprime les alias prenom.nom@domaine.
    Possible uniquement si l'utilisateur a le droit ``bureau`` ou ``aliases``
    (cf. ``info_perms``)
    """
    club, _, perms = infos_perms(request, club_id)

    if not perms or not perms["bureau"] or not perms["aliases"]:
        return forbidden(request)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            update_domaines(club.get_ldap_club(), to_delete=domaine)
        return redirect("gestion:clubs:gest_bureau", club_id=club_id)

    message = """Voulez-vous vraiment supprimer ce domaine d'alias ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer le domaine")]

    return confirmation(request, message, boutons)


def nom_uid(uid):
    """Retourne le nom complet de l'utilisateur ``uid``"""
    try:
        return PortailUser.objects.get_from_ldap(uid).get_full_name()
    except PortailUser.DoesNotExist:
        return f"<'{uid}' Inexistant>"


@access_restricted
def gest_membres(request, club_id, uid=None):
    """
    Gestion des membres qui sont dans le club ``club_id``.
    Si un uid est spécifié, il est ajouté à la liste des membres du club.

    Possible uniquement si l'utilisateur a le droit ``membres`` (cf.
    ``info_perms``)
    """
    club, infos, perms = infos_perms(request, club_id)

    if not perms or not perms["membres"]:
        return forbidden(request)

    if uid:
        etu = LdapEleve.objects.get(uid=uid)
        if club.ecole != Ecole.n7 or etu.inscritAE:
            club.ajouter_membres([uid])
            _logger.log(request.user, "A ajouté {} au club {}", uid, club_id)
            messages.success(request, f"{nom_uid(uid)} a été ajouté au club.")
        else:
            messages.error(request, f"{nom_uid(uid)} n'est pas cotisant AE.")
        return redirect("gestion:clubs:gest_membres", club_id=club_id)

    if request.method == "POST" and "envoi_action" in request.POST:
        action = request.POST.get("action")
        liste = request.POST.getlist("membre")
        noms = ", ".join([nom_uid(uid) for uid in liste])
        if action == "suppr":
            club.supprimer_membres(liste)
            _logger.log(request.user, "A retiré {} du club {}", noms, club_id)
            messages.success(
                request,
                "{} {} du club".format(noms, "a été supprimé" if len(liste) == 1 else "ont été supprimés"),
            )

        if action.startswith("aj_alias_"):
            prefixe = action[9:]
            alias = get_object_or_404(AliasGere, club=club, prefixe=prefixe, confirme=True)
            alias_ldap = alias.alias_ldap
            ajoutes = []

            for uid in liste:
                try:
                    utilisateur = PortailUser.objects.get_from_ldap(uid)
                except PortailUser.DoesNotExist:
                    continue

                adresse = f"{utilisateur.username}@{settings.EMAIL_BDE[utilisateur.ecole.id - 1]}"

                if adresse not in alias_ldap.rfc822MailMember:
                    ajoutes.append(utilisateur.get_full_name())
                    alias_ldap.rfc822MailMember.append(adresse)
                    search_login_alias(
                        alias_ldap, adresse, False
                    )  # association de l'adresse à l'utilisateur dans la BDD du portail

            alias_ldap.save()

            if ajoutes:
                _logger.log(
                    request.user,
                    "A ajouté {} à l'alias {}",
                    ", ".join(ajoutes),
                    club_id,
                )
                messages.success(
                    request,
                    "{} {} à l’alias".format(
                        ", ".join(ajoutes),
                        "a été ajouté" if len(liste) == 1 else "ont été ajoutés",
                    ),
                )
            else:
                messages.warning(request, "Aucune adresse n'a été ajouté à l'alias.")

        return redirect("gestion:clubs:gest_membres", club_id=club_id)

    c = {
        "club": club,
        "perms": perms,
        "membres": club.get_membres(infos),
        "page_membres": True,
    }

    return render(request, "gestion/clubs/gest_membres.html", c)


@access_restricted
def gest_blog(request, club_id, adresse=None):
    """
    Gestion du blog du club ``club_id``.
    Si une adresse est spécifiée, l'article lié sera modifiable.

    Possible uniquement si l'utilisateur a le droit ``blog`` (cf.
    ``info_perms``)
    """
    club, infos, perms = infos_perms(request, club_id)

    if not perms or not perms["blog"]:
        return forbidden(request)

    if not club.blog_act:
        return render(request, "gestion/clubs/gest_blog.html", {"club": club, "perms": perms})

    if adresse is not None:
        article = get_object_or_404(ArticleBlog, club=club, adresse=adresse)
        try:
            jeton = JetonEdition.objects.get(article=article)
        except JetonEdition.DoesNotExist:
            jeton = JetonEdition(article=article, editeur=request.user)
            jeton.save()
        else:
            if jeton.editeur == request.user:
                jeton.renouvellement()
            elif jeton.fin_jeton < datetime.now():
                jeton.delete()
                jeton = JetonEdition(article=article, editeur=request.user)
                jeton.save()
            else:
                msg = mark_safe("{} est en train d'éditer cet article.".format(jeton.editeur.get_link()))
                return erreur(request, msg, "Conflit d'édition", 409)
    else:
        article = None
        jeton = None

    perm_gerer_accueil = request.user.check_permission(request, "gerer_accueil")
    st_demande_accueil = False
    st_aff_accueil = False

    namespace = f"blog_club_{club.ecole.ident}_{club.ident.replace('-', '_')}"
    previs = None

    if request.method == "POST":
        form = ArticleBlogForm(request.POST, request.FILES, instance=article, namespace=namespace)
        st_demande_accueil = "demande_accueil" in request.POST
        st_aff_accueil = "aff_accueil" in request.POST

        if "previs" in request.POST:
            previs = form.get_preview()

        elif "cancel" in request.POST and jeton:
            jeton.delete()
            return redirect(article.get_absolute_url())

        elif form.is_valid():
            if article:
                article = form.save(commit=False)
                article.editeur = request.user
            else:
                article = form.save(commit=False)
                article.club = club
                article.auteur = request.user

                adresse = slugify(article.titre)[0:60]
                if ArticleBlog.objects.filter(club=club, adresse=adresse).count() > 0:
                    i = 1
                    while ArticleBlog.objects.filter(club=club, adresse=adresse + str(i)).count() > 0:
                        i += 1
                    adresse = adresse + str(i)
                article.adresse = adresse

            if perm_gerer_accueil:
                article.demande_accueil = False
                article.sur_accueil = st_aff_accueil
            elif article.sur_accueil:
                article.demande_accueil = False
            else:
                article.demande_accueil = st_demande_accueil
                if article.demande_accueil:
                    ecole_id = {
                        Ecole.inp.id: 0,
                        Ecole.n7.id: 1,
                        Ecole.ensat.id: 2,
                        Ecole.a7.id: 3,
                        Ecole.enm.id: 4,
                    }.get(request.ecole.id, None)
                    send_mail(
                        "Demande de validation d'un article " + article.club.nom,
                        "Une confirmation pour la publication d'un article est demandée : \n\nPour voir l'article : https://"
                        + settings.URLS_SITES[ecole_id]
                        + article.get_absolute_url()
                        + "\n\nPour valider : https://"
                        + settings.URLS_SITES[ecole_id]
                        + "/gestion/accueil/"
                        + article.club.ident
                        + "/"
                        + article.adresse,
                        settings.DEFAULT_FROM_EMAIL,
                        [settings.EMAIL_NEWS + "@" + settings.EMAIL_BDE[ecole_id]],
                        fail_silently=False,
                    )

            _logger.log(
                request.user,
                "A édité l'article « {} » du club {} (Ecole {})",
                article.titre,
                article.club.ident,
                article.club.ecole,
            )

            article.save()
            form.commit_attachments()

            if jeton:
                jeton.delete()

            return redirect(article.get_absolute_url())

    else:
        form = ArticleBlogForm(instance=article, namespace=namespace)
        if article is not None:
            st_demande_accueil = article.demande_accueil
            st_aff_accueil = article.sur_accueil

    c = {
        "club": club,
        "perms": perms,
        "article": article,
        "form": form,
        "previs": previs,
        "perm_gerer_accueil": perm_gerer_accueil,
        "st_demande_accueil": st_demande_accueil,
        "st_aff_accueil": st_aff_accueil,
        "articles": ArticleBlog.objects.filter(club=club).order_by("-date_post")[0:5],
        "page_blog": True,
    }

    return render(request, "gestion/clubs/gest_blog.html", c)


def infos_membre(uid):
    """Renvoie des informations sur le membre ``uid``"""
    if not uid:
        return None

    try:
        membre = LdapEleve.objects.get(uid=uid)
    except LdapEleve.DoesNotExist:
        return None

    tel = membre.homePhone
    if membre.mobile:
        tel = membre.mobile
    if tel is not None:
        tel = PortailUser.format_tel(tel, lien=False)

    return {
        "nom": membre.nom,
        "prenom": membre.prenom,
        "ddn": membre.birthdate,
        "adresse": membre.postalAddress,
        "telephone": tel,
        "mail": membre.mailAnnexe[0] if membre.mailAnnexe else membre.mailEcole,
        "ecole": membre.ecole.get().displayName,
    }


@access_restricted
def fiche_passation(request, club_id, mail=False):
    """Informations pour la fiche de passation du club ``club_uid``"""
    club, infos, perms = infos_perms(request, club_id)

    if not perms or not perms["blog"]:
        return forbidden(request)

    if club_id == "bde":
        raise Http404

    # On récupère les infos du BDE de l’école
    bde = Club.objects.par_ident(request.ecole, "bde")
    pres_bde = PortailUser.objects.get_from_ldap(bde.get_infos()["president"])
    tres_bde = PortailUser.objects.get_from_ldap(bde.get_infos()["tresorier"][0])

    # Et celles du club
    # infos = club.get_infos()
    president = infos_membre(infos["president"])
    vicePresidents = [infos_membre(vp_infos) for vp_infos in infos["vicePresident"]]
    tresorier = infos_membre(infos["tresorier"][0])
    secretaire = infos_membre(infos["secretaire"][0]) if len(infos["secretaire"]) > 0 else None

    c = {
        "club": club,
        "pres_bde": pres_bde,
        "tres_bde": tres_bde,
        "president": president,
        "vicePresidents": vicePresidents,
        "tresorier": tresorier,
        "secretaire": secretaire,
        "maintenant": datetime.now(),
        "url": "",
    }

    if mail:
        c["url"] = "https://" + request.get_host()
        return render_to_string("gestion/clubs/fiche_passation.html", c, request=request)

    return render(request, "gestion/clubs/fiche_passation.html", c)


@access_restricted
def suppr_article(request, club_id, adresse):
    club, infos, perms = infos_perms(request, club_id)

    if not perms or not perms["blog"]:
        return forbidden(request)

    if not club.blog_act:
        raise Http404

    namespace = f"blog_club_{club.ecole.ident}_{club.ident.replace('-', '_')}"
    article = get_object_or_404(ArticleBlog, club=club, adresse=adresse)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            article.contenu.disown_attachments(namespace, article.id)
            article.delete()
            _logger.log(
                request.user,
                "A supprimé l'article « {} » du club {}",
                article.titre,
                club_id,
            )
            messages.success(request, f"L'article « {article.titre} » a bien été supprimé")
        return redirect("gestion:clubs:gest_blog", club_id=club_id)

    message = f"Voules-vous vraiment supprimer l'article {article.titre} ?"
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons)


@permission_needed("gerer_clubs")
def dem_suppression(request, club_id):
    try:
        club = Club.objects.par_ident(request.ecole, club_id)
    except Club.DoesNotExist:
        raise Http404

    if request.method == "POST":
        if "conf_ok" in request.POST:
            mail_root(
                "Suppression de club",
                render_to_string("gestion/clubs/mail_suppr.txt", {"club": club}),
            )
            _logger.log(request.user, "A demandé la suppression du clubs {}", club_id)
            messages.success(request, "Demande de suppression du club prise en compte.")
        return redirect("gestion:clubs:gest_bureau", club_id=club_id)

    message = """Voulez-vous vraiment demander la suppression de ce club ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons)


class NouvClubEncoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, PortailUser):
            return o.username

        return JSONEncoder.default(self, o)


@permission_needed("gerer_clubs")
def nouveau(request):
    if request.method == "POST":
        form = NouvClubForm(request.POST)
        if form.is_valid():
            donnees = form.cleaned_data
            donnees["ecole"] = request.ecole.ident
            donnees_enc = b64encode(NouvClubEncoder().encode(donnees).encode("utf8")).decode("utf8")
            mail_root(
                "Nouveau club",
                render_to_string(
                    "gestion/clubs/mail_creat.txt",
                    {
                        "donnees_enc": donnees_enc,
                        "donnees": donnees,
                        "ecole": request.ecole,
                    },
                ),
            )
            _logger.log(request.user, "A demandé la création du club {}", donnees["nom"])
            messages.success(request, "Demande de création du club prise en compte.")
            return redirect("gestion:clubs:liste")
    else:
        form = NouvClubForm()

    return render(request, "gestion/clubs/nouveau.html", {"form": form})


@access_restricted
def gest_edt(request, club_id, event_id=None):
    """
    Gestion du calendrier du club ``club_id`` :
    * ajout d'un événement
    * modification de l'événement ``event_id``
    """
    club, infos, perms = infos_perms(request, club_id)

    if not perms or not perms["blog"]:
        return forbidden(request)

    if event_id is not None:
        event = get_object_or_404(EdtClubs, id=event_id)
        event_mod = True
    else:
        event = None
        event_mod = False

    if request.method == "POST":
        form = EdtClubsForm(request.POST, instance=event)
        if form.is_valid():
            event = form.save(commit=False)
            event.club = club.ident_groupe
            event.save()
            if event_mod:
                messages.success(request, "Évènement modifié avec succès.")
            else:
                _logger.log(
                    request.user,
                    "A créé l'événement {} pour le club {}",
                    event.topic,
                    club_id,
                )
                messages.success(request, "Évènement créé avec succès.")
            return redirect("gestion:clubs:gest_edt", club_id)
    else:
        form = EdtClubsForm(instance=event)

    club_cn = club.get_ldap_club().cn
    events = EdtClubs.objects.filter(club=club_cn)

    c = {
        "club": club,
        "form": form,
        "events": events,
        "perms": perms,
        "page_edt": True,
    }

    return render(request, "gestion/clubs/gest_edt.html", c)


@access_restricted
def del_event(request, event_id, club_id):
    club, infos, perms = infos_perms(request, club_id)
    event = get_object_or_404(EdtClubs, id=event_id)

    if not perms or not perms["blog"]:
        return forbidden(request)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            event.delete()
            cache.clear()
            _logger.log(
                request.user,
                "A supprimé l'événement {} du club {}",
                event.topic,
                club_id,
            )
            messages.success(request, "Évènement supprimé avec succès.")
        return redirect("gestion:clubs:gest_edt", club_id)

    message = "Êtes-vous sûr(e) de vouloir supprimer cet événement ?"
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer un événement")
